/*
 * Copyright (C) 2008,2009 Sebastian Pölsterl
 *
 * This file is part of GNOME DVB Daemon.
 *
 * GNOME DVB Daemon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * GNOME DVB Daemon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNOME DVB Daemon.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gee.h>
#include <gio/gio.h>
#include <dbus/dbus-glib-lowlevel.h>
#include <dbus/dbus-glib.h>
#include <dbus/dbus.h>


#define DVB_TYPE_ID_BUS_DEVICE_GROUP (dvb_id_bus_device_group_get_type ())
#define DVB_ID_BUS_DEVICE_GROUP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DVB_TYPE_ID_BUS_DEVICE_GROUP, DVBIDBusDeviceGroup))
#define DVB_IS_ID_BUS_DEVICE_GROUP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DVB_TYPE_ID_BUS_DEVICE_GROUP))
#define DVB_ID_BUS_DEVICE_GROUP_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), DVB_TYPE_ID_BUS_DEVICE_GROUP, DVBIDBusDeviceGroupIface))

typedef struct _DVBIDBusDeviceGroup DVBIDBusDeviceGroup;
typedef struct _DVBIDBusDeviceGroupIface DVBIDBusDeviceGroupIface;

#define DVB_TYPE_DEVICE_GROUP (dvb_device_group_get_type ())
#define DVB_DEVICE_GROUP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DVB_TYPE_DEVICE_GROUP, DVBDeviceGroup))
#define DVB_DEVICE_GROUP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DVB_TYPE_DEVICE_GROUP, DVBDeviceGroupClass))
#define DVB_IS_DEVICE_GROUP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DVB_TYPE_DEVICE_GROUP))
#define DVB_IS_DEVICE_GROUP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DVB_TYPE_DEVICE_GROUP))
#define DVB_DEVICE_GROUP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DVB_TYPE_DEVICE_GROUP, DVBDeviceGroupClass))

typedef struct _DVBDeviceGroup DVBDeviceGroup;
typedef struct _DVBDeviceGroupClass DVBDeviceGroupClass;
typedef struct _DVBDeviceGroupPrivate DVBDeviceGroupPrivate;

#define DVB_TYPE_DEVICE (dvb_device_get_type ())
#define DVB_DEVICE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DVB_TYPE_DEVICE, DVBDevice))
#define DVB_DEVICE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DVB_TYPE_DEVICE, DVBDeviceClass))
#define DVB_IS_DEVICE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DVB_TYPE_DEVICE))
#define DVB_IS_DEVICE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DVB_TYPE_DEVICE))
#define DVB_DEVICE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DVB_TYPE_DEVICE, DVBDeviceClass))

typedef struct _DVBDevice DVBDevice;
typedef struct _DVBDeviceClass DVBDeviceClass;

#define DVB_TYPE_RECORDER (dvb_recorder_get_type ())
#define DVB_RECORDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DVB_TYPE_RECORDER, DVBRecorder))
#define DVB_RECORDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DVB_TYPE_RECORDER, DVBRecorderClass))
#define DVB_IS_RECORDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DVB_TYPE_RECORDER))
#define DVB_IS_RECORDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DVB_TYPE_RECORDER))
#define DVB_RECORDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DVB_TYPE_RECORDER, DVBRecorderClass))

typedef struct _DVBRecorder DVBRecorder;
typedef struct _DVBRecorderClass DVBRecorderClass;

#define DVB_TYPE_EPG_SCANNER (dvb_epg_scanner_get_type ())
#define DVB_EPG_SCANNER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DVB_TYPE_EPG_SCANNER, DVBEPGScanner))
#define DVB_EPG_SCANNER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DVB_TYPE_EPG_SCANNER, DVBEPGScannerClass))
#define DVB_IS_EPG_SCANNER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DVB_TYPE_EPG_SCANNER))
#define DVB_IS_EPG_SCANNER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DVB_TYPE_EPG_SCANNER))
#define DVB_EPG_SCANNER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DVB_TYPE_EPG_SCANNER, DVBEPGScannerClass))

typedef struct _DVBEPGScanner DVBEPGScanner;
typedef struct _DVBEPGScannerClass DVBEPGScannerClass;

#define DVB_TYPE_CHANNEL_FACTORY (dvb_channel_factory_get_type ())
#define DVB_CHANNEL_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DVB_TYPE_CHANNEL_FACTORY, DVBChannelFactory))
#define DVB_CHANNEL_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DVB_TYPE_CHANNEL_FACTORY, DVBChannelFactoryClass))
#define DVB_IS_CHANNEL_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DVB_TYPE_CHANNEL_FACTORY))
#define DVB_IS_CHANNEL_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DVB_TYPE_CHANNEL_FACTORY))
#define DVB_CHANNEL_FACTORY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DVB_TYPE_CHANNEL_FACTORY, DVBChannelFactoryClass))

typedef struct _DVBChannelFactory DVBChannelFactory;
typedef struct _DVBChannelFactoryClass DVBChannelFactoryClass;

#define DVB_TYPE_ADAPTER_TYPE (dvb_adapter_type_get_type ())

#define DVB_TYPE_CHANNEL_LIST (dvb_channel_list_get_type ())
#define DVB_CHANNEL_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DVB_TYPE_CHANNEL_LIST, DVBChannelList))
#define DVB_CHANNEL_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DVB_TYPE_CHANNEL_LIST, DVBChannelListClass))
#define DVB_IS_CHANNEL_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DVB_TYPE_CHANNEL_LIST))
#define DVB_IS_CHANNEL_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DVB_TYPE_CHANNEL_LIST))
#define DVB_CHANNEL_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DVB_TYPE_CHANNEL_LIST, DVBChannelListClass))

typedef struct _DVBChannelList DVBChannelList;
typedef struct _DVBChannelListClass DVBChannelListClass;

#define DVB_TYPE_MANAGER (dvb_manager_get_type ())
#define DVB_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DVB_TYPE_MANAGER, DVBManager))
#define DVB_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DVB_TYPE_MANAGER, DVBManagerClass))
#define DVB_IS_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DVB_TYPE_MANAGER))
#define DVB_IS_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DVB_TYPE_MANAGER))
#define DVB_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DVB_TYPE_MANAGER, DVBManagerClass))

typedef struct _DVBManager DVBManager;
typedef struct _DVBManagerClass DVBManagerClass;

#define DVB_TYPE_CONFIG_STORE (dvb_config_store_get_type ())
#define DVB_CONFIG_STORE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DVB_TYPE_CONFIG_STORE, DVBConfigStore))
#define DVB_IS_CONFIG_STORE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DVB_TYPE_CONFIG_STORE))
#define DVB_CONFIG_STORE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), DVB_TYPE_CONFIG_STORE, DVBConfigStoreIface))

typedef struct _DVBConfigStore DVBConfigStore;
typedef struct _DVBConfigStoreIface DVBConfigStoreIface;

#define DVB_TYPE_CHANNEL (dvb_channel_get_type ())
#define DVB_CHANNEL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DVB_TYPE_CHANNEL, DVBChannel))
#define DVB_CHANNEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DVB_TYPE_CHANNEL, DVBChannelClass))
#define DVB_IS_CHANNEL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DVB_TYPE_CHANNEL))
#define DVB_IS_CHANNEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DVB_TYPE_CHANNEL))
#define DVB_CHANNEL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DVB_TYPE_CHANNEL, DVBChannelClass))

typedef struct _DVBChannel DVBChannel;
typedef struct _DVBChannelClass DVBChannelClass;

#define DVB_TYPE_SCHEDULE (dvb_schedule_get_type ())
#define DVB_SCHEDULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DVB_TYPE_SCHEDULE, DVBSchedule))
#define DVB_SCHEDULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DVB_TYPE_SCHEDULE, DVBScheduleClass))
#define DVB_IS_SCHEDULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DVB_TYPE_SCHEDULE))
#define DVB_IS_SCHEDULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DVB_TYPE_SCHEDULE))
#define DVB_SCHEDULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DVB_TYPE_SCHEDULE, DVBScheduleClass))

typedef struct _DVBSchedule DVBSchedule;
typedef struct _DVBScheduleClass DVBScheduleClass;
typedef struct _DBusObjectVTable _DBusObjectVTable;

struct _DVBIDBusDeviceGroupIface {
	GTypeInterface parent_iface;
	char* (*GetType) (DVBIDBusDeviceGroup* self);
	char* (*GetRecorder) (DVBIDBusDeviceGroup* self);
	gboolean (*AddDevice) (DVBIDBusDeviceGroup* self, guint adapter, guint frontend);
	gboolean (*RemoveDevice) (DVBIDBusDeviceGroup* self, guint adapter, guint frontend);
	char* (*GetChannelList) (DVBIDBusDeviceGroup* self);
	char* (*GetName) (DVBIDBusDeviceGroup* self);
	gboolean (*SetName) (DVBIDBusDeviceGroup* self, const char* name);
	char** (*GetMembers) (DVBIDBusDeviceGroup* self, int* result_length1);
	char* (*GetSchedule) (DVBIDBusDeviceGroup* self, guint channel_sid);
	char* (*GetRecordingsDirectory) (DVBIDBusDeviceGroup* self);
	gboolean (*SetRecordingsDirectory) (DVBIDBusDeviceGroup* self, const char* location);
};

/**
     * A group of devices that share the same settings
     * (list of channels, recordings dir)
     */
struct _DVBDeviceGroup {
	GObject parent_instance;
	DVBDeviceGroupPrivate * priv;
};

struct _DVBDeviceGroupClass {
	GObjectClass parent_class;
};

struct _DVBDeviceGroupPrivate {
	guint _Id;
	char* _Name;
	DVBDevice* _reference_device;
	GeeSet* devices;
	GStaticRecMutex __lock_devices;
	DVBRecorder* _recorder;
	DVBEPGScanner* _epgscanner;
	DVBChannelFactory* _channelfactory;
	GeeHashSet* schedules;
};

typedef enum  {
	DVB_ADAPTER_TYPE_UNKNOWN,
	DVB_ADAPTER_TYPE_DVB_T,
	DVB_ADAPTER_TYPE_DVB_S,
	DVB_ADAPTER_TYPE_DVB_C
} DVBAdapterType;

struct _DVBConfigStoreIface {
	GTypeInterface parent_iface;
	GeeList* (*get_all_device_groups) (DVBConfigStore* self);
	gboolean (*add_device_group) (DVBConfigStore* self, DVBDeviceGroup* dev_group);
	gboolean (*remove_device_group) (DVBConfigStore* self, DVBDeviceGroup* devgroup);
	gboolean (*add_device_to_group) (DVBConfigStore* self, DVBDevice* dev, DVBDeviceGroup* devgroup);
	gboolean (*remove_device_from_group) (DVBConfigStore* self, DVBDevice* dev, DVBDeviceGroup* devgroup);
	gboolean (*update_from_group) (DVBConfigStore* self, DVBDeviceGroup* devgroup);
};

struct _DBusObjectVTable {
	void (*register_object) (DBusConnection*, const char*, void*);
};


static gpointer dvb_device_group_parent_class = NULL;
static DVBIDBusDeviceGroupIface* dvb_device_group_dvb_id_bus_device_group_parent_iface = NULL;
static GeeIterableIface* dvb_device_group_gee_iterable_parent_iface = NULL;

GType dvb_id_bus_device_group_get_type (void);
GType dvb_device_group_get_type (void);
GType dvb_device_get_type (void);
GType dvb_recorder_get_type (void);
GType dvb_epg_scanner_get_type (void);
GType dvb_channel_factory_get_type (void);
#define DVB_DEVICE_GROUP_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), DVB_TYPE_DEVICE_GROUP, DVBDeviceGroupPrivate))
enum  {
	DVB_DEVICE_GROUP_DUMMY_PROPERTY,
	DVB_DEVICE_GROUP_SIZE,
	DVB_DEVICE_GROUP_ID,
	DVB_DEVICE_GROUP_CHANNELS,
	DVB_DEVICE_GROUP_RECORDINGS_DIRECTORY,
	DVB_DEVICE_GROUP_TYPE,
	DVB_DEVICE_GROUP_RECORDER,
	DVB_DEVICE_GROUP_EPGSCANNER,
	DVB_DEVICE_GROUP_CHANNEL_FACTORY,
	DVB_DEVICE_GROUP_NAME,
	DVB_DEVICE_GROUP_REFERENCE_DEVICE
};
static void dvb_device_group_set_Id (DVBDeviceGroup* self, guint value);
static void dvb_device_group_set_reference_device (DVBDeviceGroup* self, DVBDevice* value);
DVBEPGScanner* dvb_epg_scanner_new (DVBDeviceGroup* device);
DVBEPGScanner* dvb_epg_scanner_construct (GType object_type, DVBDeviceGroup* device);
gboolean dvb_device_group_register_channel_list (DVBDeviceGroup* self);
gboolean dvb_device_group_register_recorder (DVBDeviceGroup* self);
DVBDeviceGroup* dvb_device_group_new (guint id, DVBDevice* reference_device, gboolean with_epg_scanner);
DVBDeviceGroup* dvb_device_group_construct (GType object_type, guint id, DVBDevice* reference_device, gboolean with_epg_scanner);
guint dvb_device_group_get_Id (DVBDeviceGroup* self);
void dvb_epg_scanner_stop (DVBEPGScanner* self);
void dvb_recorder_stop (DVBRecorder* self);
void dvb_channel_factory_destroy (DVBChannelFactory* self);
void dvb_device_group_destroy (DVBDeviceGroup* self);
DVBDevice* dvb_device_new (guint adapter, guint frontend, gboolean get_type_and_name);
DVBDevice* dvb_device_construct (GType object_type, guint adapter, guint frontend, gboolean get_type_and_name);
gboolean dvb_device_group_add (DVBDeviceGroup* self, DVBDevice* device);
void dvb_device_group_create_and_add_device (DVBDeviceGroup* self, guint adapter, guint frontend);
GType dvb_adapter_type_get_type (void);
DVBAdapterType dvb_device_get_Type (DVBDevice* self);
DVBAdapterType dvb_device_group_get_Type (DVBDeviceGroup* self);
DVBDevice* dvb_device_group_get_reference_device (DVBDeviceGroup* self);
GType dvb_channel_list_get_type (void);
DVBChannelList* dvb_device_get_Channels (DVBDevice* self);
void dvb_device_set_Channels (DVBDevice* self, DVBChannelList* value);
GFile* dvb_device_get_RecordingsDirectory (DVBDevice* self);
void dvb_device_set_RecordingsDirectory (DVBDevice* self, GFile* value);
gboolean dvb_device_group_contains (DVBDeviceGroup* self, DVBDevice* device);
gboolean dvb_device_group_remove (DVBDeviceGroup* self, DVBDevice* device);
gboolean dvb_device_is_busy (DVBDevice* self);
DVBDevice* dvb_device_group_get_next_free_device (DVBDeviceGroup* self);
static char* dvb_device_group_real_GetType (DVBIDBusDeviceGroup* base);
GType dvb_manager_get_type (void);
DVBManager* dvb_manager_get_instance (void);
gboolean dvb_manager_device_is_in_any_group (DVBManager* self, DVBDevice* device);
GType dvb_config_store_get_type (void);
DVBConfigStore* dvb_factory_get_config_store (void);
gboolean dvb_config_store_add_device_to_group (DVBConfigStore* self, DVBDevice* dev, DVBDeviceGroup* devgroup);
static gboolean dvb_device_group_real_AddDevice (DVBIDBusDeviceGroup* base, guint adapter, guint frontend);
#define DVB_CONSTANTS_DBUS_RECORDER_PATH "/org/gnome/DVB/DeviceGroup/%u/Recorder"
static char* dvb_device_group_real_GetRecorder (DVBIDBusDeviceGroup* base);
DVBRecorder* dvb_device_group_get_recorder (DVBDeviceGroup* self);
DBusGConnection* dvb_utils_get_dbus_connection (void);
DVBEPGScanner* dvb_device_group_get_epgscanner (DVBDeviceGroup* self);
gboolean dvb_config_store_remove_device_from_group (DVBConfigStore* self, DVBDevice* dev, DVBDeviceGroup* devgroup);
gint dvb_device_group_get_size (DVBDeviceGroup* self);
gboolean dvb_epg_scanner_start (DVBEPGScanner* self);
static gboolean dvb_device_group_real_RemoveDevice (DVBIDBusDeviceGroup* base, guint adapter, guint frontend);
const char* dvb_device_group_get_Name (DVBDeviceGroup* self);
static char* dvb_device_group_real_GetName (DVBIDBusDeviceGroup* base);
void dvb_device_group_set_Name (DVBDeviceGroup* self, const char* value);
gboolean dvb_config_store_update_from_group (DVBConfigStore* self, DVBDeviceGroup* devgroup);
static gboolean dvb_device_group_real_SetName (DVBIDBusDeviceGroup* base, const char* name);
#define DVB_CONSTANTS_DBUS_CHANNEL_LIST_PATH "/org/gnome/DVB/DeviceGroup/%u/ChannelList"
static char* dvb_device_group_real_GetChannelList (DVBIDBusDeviceGroup* base);
DVBChannelList* dvb_device_group_get_Channels (DVBDeviceGroup* self);
#define DVB_CONSTANTS_DVB_DEVICE_PATH "/dev/dvb/adapter%u/frontend%u"
guint dvb_device_get_Adapter (DVBDevice* self);
guint dvb_device_get_Frontend (DVBDevice* self);
static char** dvb_device_group_real_GetMembers (DVBIDBusDeviceGroup* base, int* result_length1);
gboolean dvb_channel_list_contains (DVBChannelList* self, guint sid);
#define DVB_CONSTANTS_DBUS_SCHEDULE_PATH "/org/gnome/DVB/DeviceGroup/%u/Schedule/%u"
GType dvb_channel_get_type (void);
DVBChannel* dvb_channel_list_get_channel (DVBChannelList* self, guint sid);
GType dvb_schedule_get_type (void);
DVBSchedule* dvb_channel_get_Schedule (DVBChannel* self);
static char* dvb_device_group_real_GetSchedule (DVBIDBusDeviceGroup* base, guint channel_sid);
GFile* dvb_device_group_get_RecordingsDirectory (DVBDeviceGroup* self);
static char* dvb_device_group_real_GetRecordingsDirectory (DVBIDBusDeviceGroup* base);
void dvb_device_group_set_RecordingsDirectory (DVBDeviceGroup* self, GFile* value);
static gboolean dvb_device_group_real_SetRecordingsDirectory (DVBIDBusDeviceGroup* base, const char* location);
static GType dvb_device_group_real_get_element_type (GeeIterable* base);
static GeeIterator* dvb_device_group_real_iterator (GeeIterable* base);
static void dvb_device_group_update_all_devices (DVBDeviceGroup* self);
DVBChannelFactory* dvb_device_group_get_channel_factory (DVBDeviceGroup* self);
guint dvb_device_hash (DVBDevice* device);
static guint _dvb_device_hash_ghash_func (void* key);
gboolean dvb_device_equal (DVBDevice* dev1, DVBDevice* dev2);
static gboolean _dvb_device_equal_gequal_func (void* a, void* b);
DVBChannelFactory* dvb_channel_factory_new (DVBDeviceGroup* devgroup);
DVBChannelFactory* dvb_channel_factory_construct (GType object_type, DVBDeviceGroup* devgroup);
DVBRecorder* dvb_recorder_new (DVBDeviceGroup* dev);
DVBRecorder* dvb_recorder_construct (GType object_type, DVBDeviceGroup* dev);
static GObject * dvb_device_group_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
void dvb_device_group_dbus_register_object (DBusConnection* connection, const char* path, void* object);
void _dvb_device_group_dbus_unregister (DBusConnection* connection, void* user_data);
DBusHandlerResult dvb_device_group_dbus_message (DBusConnection* connection, DBusMessage* message, void* object);
static DBusMessage* _dbus_dvb_device_group_introspect (DVBDeviceGroup* self, DBusConnection* connection, DBusMessage* message);
static void dvb_device_group_finalize (GObject* obj);
static void dvb_device_group_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void dvb_device_group_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_dbus_register_object (DBusConnection* connection, const char* path, void* object);
static void _vala_dbus_unregister_object (gpointer connection, GObject* object);

static const DBusObjectPathVTable _dvb_device_group_dbus_path_vtable = {_dvb_device_group_dbus_unregister, dvb_device_group_dbus_message};
static const _DBusObjectVTable _dvb_device_group_dbus_vtable = {dvb_device_group_dbus_register_object};


/**
         * @id: ID of group
         * @reference_device: All devices of this group will inherit
         * the settings from this device
         * @with_epg_scanner: Whether to provide an EPG scanner
         */
DVBDeviceGroup* dvb_device_group_construct (GType object_type, guint id, DVBDevice* reference_device, gboolean with_epg_scanner) {
	GParameter * __params;
	GParameter * __params_it;
	DVBDeviceGroup * self;
	g_return_val_if_fail (reference_device != NULL, NULL);
	__params = g_new0 (GParameter, 2);
	__params_it = __params;
	__params_it->name = "Id";
	g_value_init (&__params_it->value, G_TYPE_UINT);
	g_value_set_uint (&__params_it->value, id);
	__params_it++;
	__params_it->name = "reference-device";
	g_value_init (&__params_it->value, DVB_TYPE_DEVICE);
	g_value_set_object (&__params_it->value, reference_device);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	if (with_epg_scanner) {
		DVBEPGScanner* _tmp0_;
		_tmp0_ = NULL;
		self->priv->_epgscanner = (_tmp0_ = dvb_epg_scanner_new (self), (self->priv->_epgscanner == NULL) ? NULL : (self->priv->_epgscanner = (g_object_unref (self->priv->_epgscanner), NULL)), _tmp0_);
	} else {
		DVBEPGScanner* _tmp1_;
		_tmp1_ = NULL;
		self->priv->_epgscanner = (_tmp1_ = NULL, (self->priv->_epgscanner == NULL) ? NULL : (self->priv->_epgscanner = (g_object_unref (self->priv->_epgscanner), NULL)), _tmp1_);
	}
	dvb_device_group_register_channel_list (self);
	dvb_device_group_register_recorder (self);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


DVBDeviceGroup* dvb_device_group_new (guint id, DVBDevice* reference_device, gboolean with_epg_scanner) {
	return dvb_device_group_construct (DVB_TYPE_DEVICE_GROUP, id, reference_device, with_epg_scanner);
}


void dvb_device_group_destroy (DVBDeviceGroup* self) {
	g_return_if_fail (self != NULL);
	g_debug ("DeviceGroup.vala:99: Destroying group %u", self->priv->_Id);
	if (self->priv->_epgscanner != NULL) {
		dvb_epg_scanner_stop (self->priv->_epgscanner);
	}
	dvb_recorder_stop (self->priv->_recorder);
	dvb_channel_factory_destroy (self->priv->_channelfactory);
	gee_collection_clear ((GeeCollection*) self->priv->schedules);
	g_static_rec_mutex_lock (&self->priv->__lock_devices);
	{
		gee_collection_clear ((GeeCollection*) self->priv->devices);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_devices);
}


/**
         * @adapter: Number of the device's adapter
         * @frontend: Number of the device's frontend
         *
         * Creates a new device first and adds it to the group.
         * The new device inherits the settings from the reference
         * device.
         */
void dvb_device_group_create_and_add_device (DVBDeviceGroup* self, guint adapter, guint frontend) {
	DVBDevice* new_dev;
	g_return_if_fail (self != NULL);
	new_dev = dvb_device_new (adapter, frontend, TRUE);
	dvb_device_group_add (self, new_dev);
	(new_dev == NULL) ? NULL : (new_dev = (g_object_unref (new_dev), NULL));
}


/**
         * Add device to group. The device's settings will be overridden
         * with those of the reference device.
         */
gboolean dvb_device_group_add (DVBDeviceGroup* self, DVBDevice* device) {
	gboolean result;
	gboolean _result_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (device != NULL, FALSE);
	if (dvb_device_get_Type (device) != dvb_device_group_get_Type (self)) {
		g_warning ("DeviceGroup.vala:130: Cannot add device, because it is not of same type");
		result = FALSE;
		return result;
	}
	_result_ = FALSE;
	g_static_rec_mutex_lock (&self->priv->__lock_devices);
	{
		/* Set settings from reference device*/
		dvb_device_set_Channels (device, dvb_device_get_Channels (self->priv->_reference_device));
		dvb_device_set_RecordingsDirectory (device, dvb_device_get_RecordingsDirectory (self->priv->_reference_device));
		_result_ = gee_collection_add ((GeeCollection*) self->priv->devices, device);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_devices);
	result = _result_;
	return result;
}


gboolean dvb_device_group_contains (DVBDeviceGroup* self, DVBDevice* device) {
	gboolean result;
	gboolean _result_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (device != NULL, FALSE);
	_result_ = FALSE;
	g_static_rec_mutex_lock (&self->priv->__lock_devices);
	{
		_result_ = gee_collection_contains ((GeeCollection*) self->priv->devices, device);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_devices);
	result = _result_;
	return result;
}


gboolean dvb_device_group_remove (DVBDeviceGroup* self, DVBDevice* device) {
	gboolean result;
	gboolean _result_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (device != NULL, FALSE);
	_result_ = FALSE;
	g_static_rec_mutex_lock (&self->priv->__lock_devices);
	{
		_result_ = gee_collection_remove ((GeeCollection*) self->priv->devices, device);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_devices);
	result = _result_;
	return result;
}


/**
         * Get first device that isn't busy.
         * If all devices are busy NULL is returned.
         */
DVBDevice* dvb_device_group_get_next_free_device (DVBDeviceGroup* self) {
	DVBDevice* result;
	DVBDevice* _result_;
	g_return_val_if_fail (self != NULL, NULL);
	_result_ = NULL;
	g_static_rec_mutex_lock (&self->priv->__lock_devices);
	{
		{
			GeeIterator* _dev_it;
			_dev_it = gee_iterable_iterator ((GeeIterable*) self->priv->devices);
			while (TRUE) {
				DVBDevice* dev;
				if (!gee_iterator_next (_dev_it)) {
					break;
				}
				dev = (DVBDevice*) gee_iterator_get (_dev_it);
				if (!dvb_device_is_busy (dev)) {
					DVBDevice* _tmp1_;
					DVBDevice* _tmp0_;
					_tmp1_ = NULL;
					_tmp0_ = NULL;
					_result_ = (_tmp1_ = (_tmp0_ = dev, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (_result_ == NULL) ? NULL : (_result_ = (g_object_unref (_result_), NULL)), _tmp1_);
					(dev == NULL) ? NULL : (dev = (g_object_unref (dev), NULL));
					break;
				}
				(dev == NULL) ? NULL : (dev = (g_object_unref (dev), NULL));
			}
			(_dev_it == NULL) ? NULL : (_dev_it = (g_object_unref (_dev_it), NULL));
		}
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_devices);
	result = _result_;
	return result;
}


/**
         * @returns: Name of adapter type the group holds
         * or an empty string when group with given id doesn't exist.
         */
static char* dvb_device_group_real_GetType (DVBIDBusDeviceGroup* base) {
	DVBDeviceGroup * self;
	char* result;
	char* type_str;
	self = (DVBDeviceGroup*) base;
	type_str = NULL;
	switch (dvb_device_group_get_Type (self)) {
		case DVB_ADAPTER_TYPE_DVB_T:
		{
			char* _tmp0_;
			_tmp0_ = NULL;
			type_str = (_tmp0_ = g_strdup ("DVB-T"), type_str = (g_free (type_str), NULL), _tmp0_);
			break;
		}
		case DVB_ADAPTER_TYPE_DVB_S:
		{
			char* _tmp1_;
			_tmp1_ = NULL;
			type_str = (_tmp1_ = g_strdup ("DVB-S"), type_str = (g_free (type_str), NULL), _tmp1_);
			break;
		}
		case DVB_ADAPTER_TYPE_DVB_C:
		{
			char* _tmp2_;
			_tmp2_ = NULL;
			type_str = (_tmp2_ = g_strdup ("DVB-C"), type_str = (g_free (type_str), NULL), _tmp2_);
			break;
		}
		default:
		{
			char* _tmp3_;
			_tmp3_ = NULL;
			type_str = (_tmp3_ = g_strdup (""), type_str = (g_free (type_str), NULL), _tmp3_);
			break;
		}
	}
	result = type_str;
	return result;
}


/**
         * @adapter: Number of the device's adapter
         * @frontend: Number of the device's frontend
         * @returns: TRUE when the device has been registered successfully
         *
         * Creates a new device and adds it to the DeviceGroup.
         * The new device will inherit all settings from the group's
         * reference device.
         */
static gboolean dvb_device_group_real_AddDevice (DVBIDBusDeviceGroup* base, guint adapter, guint frontend) {
	DVBDeviceGroup * self;
	gboolean result;
	DVBDevice* device;
	DVBManager* _tmp0_;
	DVBManager* manager;
	guint group_id;
	self = (DVBDeviceGroup*) base;
	/* When the device is already registered we
	 might see some errors if the device is
	 currently in use*/
	device = dvb_device_new (adapter, frontend, TRUE);
	if (device == NULL) {
		result = FALSE;
		(device == NULL) ? NULL : (device = (g_object_unref (device), NULL));
		return result;
	}
	_tmp0_ = NULL;
	manager = (_tmp0_ = dvb_manager_get_instance (), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	if (dvb_manager_device_is_in_any_group (manager, device)) {
		g_debug ("Device with adapter %u, frontend %u is" "already part of a group", adapter, frontend);
		result = FALSE;
		(device == NULL) ? NULL : (device = (g_object_unref (device), NULL));
		(manager == NULL) ? NULL : (manager = (g_object_unref (manager), NULL));
		return result;
	}
	group_id = self->priv->_Id;
	g_debug ("DeviceGroup.vala:219: Adding device with adapter %u, frontend %u to group %u", adapter, frontend, group_id);
	if (dvb_device_group_add (self, device)) {
		dvb_config_store_add_device_to_group (dvb_factory_get_config_store (), device, self);
		g_signal_emit_by_name ((DVBIDBusDeviceGroup*) self, "device-added", adapter, frontend);
		result = TRUE;
		(device == NULL) ? NULL : (device = (g_object_unref (device), NULL));
		(manager == NULL) ? NULL : (manager = (g_object_unref (manager), NULL));
		return result;
	}
	result = FALSE;
	(device == NULL) ? NULL : (device = (g_object_unref (device), NULL));
	(manager == NULL) ? NULL : (manager = (g_object_unref (manager), NULL));
	return result;
}


/**
         * @returns: Object path of the device's recorder
         * 
         * Returns the object path to the device's recorder.
         */
static char* dvb_device_group_real_GetRecorder (DVBIDBusDeviceGroup* base) {
	DVBDeviceGroup * self;
	char* result;
	char* _tmp0_;
	char* _tmp1_;
	self = (DVBDeviceGroup*) base;
	_tmp0_ = NULL;
	_tmp1_ = NULL;
	result = (_tmp1_ = g_strdup (_tmp0_ = g_strdup_printf (DVB_CONSTANTS_DBUS_RECORDER_PATH, self->priv->_Id)), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
	return result;
}


gboolean dvb_device_group_register_recorder (DVBDeviceGroup* self) {
	gboolean result;
	DVBRecorder* _tmp0_;
	DVBRecorder* recorder;
	DBusGConnection* conn;
	char* path;
	g_return_val_if_fail (self != NULL, FALSE);
	g_debug ("DeviceGroup.vala:245: Creating new Recorder D-Bus service for group %u", self->priv->_Id);
	_tmp0_ = NULL;
	recorder = (_tmp0_ = dvb_device_group_get_recorder (self), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	conn = dvb_utils_get_dbus_connection ();
	if (conn == NULL) {
		result = FALSE;
		(recorder == NULL) ? NULL : (recorder = (g_object_unref (recorder), NULL));
		(conn == NULL) ? NULL : (conn = (dbus_g_connection_unref (conn), NULL));
		return result;
	}
	path = g_strdup_printf (DVB_CONSTANTS_DBUS_RECORDER_PATH, self->priv->_Id);
	_vala_dbus_register_object (dbus_g_connection_get_connection (conn), path, (GObject*) recorder);
	result = TRUE;
	(recorder == NULL) ? NULL : (recorder = (g_object_unref (recorder), NULL));
	(conn == NULL) ? NULL : (conn = (dbus_g_connection_unref (conn), NULL));
	path = (g_free (path), NULL);
	return result;
}


/**
         * @adapter: Number of the device's adapter
         * @frontend: Number of the device's frontend
         * @returns: TRUE when device has been removed successfully
         *
         * Removes the device from the group. If the group contains
         * no devices after the removal it's removed as well.
         */
static gboolean dvb_device_group_real_RemoveDevice (DVBIDBusDeviceGroup* base, guint adapter, guint frontend) {
	DVBDeviceGroup * self;
	gboolean result;
	DVBDevice* dev;
	self = (DVBDeviceGroup*) base;
	dev = dvb_device_new (adapter, frontend, FALSE);
	if (dvb_device_group_contains (self, dev)) {
		if (dvb_device_group_remove (self, dev)) {
			gboolean _tmp0_;
			/* Stop epgscanner, because it might use the
			 device we want to unregister*/
			if (dvb_device_group_get_epgscanner (self) != NULL) {
				dvb_epg_scanner_stop (dvb_device_group_get_epgscanner (self));
			}
			dvb_config_store_remove_device_from_group (dvb_factory_get_config_store (), dev, self);
			_tmp0_ = FALSE;
			if (dvb_device_group_get_size (self) > 0) {
				_tmp0_ = dvb_device_group_get_epgscanner (self) != NULL;
			} else {
				_tmp0_ = FALSE;
			}
			/* Group has no devices anymore, delete it*/
			if (_tmp0_) {
				/* We still have a device, start EPG scanner again*/
				dvb_epg_scanner_start (dvb_device_group_get_epgscanner (self));
			}
			g_signal_emit_by_name ((DVBIDBusDeviceGroup*) self, "device-removed", adapter, frontend);
			result = TRUE;
			(dev == NULL) ? NULL : (dev = (g_object_unref (dev), NULL));
			return result;
		}
	}
	result = FALSE;
	(dev == NULL) ? NULL : (dev = (g_object_unref (dev), NULL));
	return result;
}


/**
         * @returns: Name of the device group
         */
static char* dvb_device_group_real_GetName (DVBIDBusDeviceGroup* base) {
	DVBDeviceGroup * self;
	char* result;
	const char* _tmp0_;
	self = (DVBDeviceGroup*) base;
	_tmp0_ = NULL;
	result = (_tmp0_ = self->priv->_Name, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	return result;
}


/**
         * @name: Name of the group
         * @returns: TRUE on success
         */
static gboolean dvb_device_group_real_SetName (DVBIDBusDeviceGroup* base, const char* name) {
	DVBDeviceGroup * self;
	gboolean result;
	DVBConfigStore* _tmp0_;
	DVBConfigStore* config;
	self = (DVBDeviceGroup*) base;
	g_return_val_if_fail (name != NULL, FALSE);
	dvb_device_group_set_Name (self, name);
	_tmp0_ = NULL;
	config = (_tmp0_ = dvb_factory_get_config_store (), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	dvb_config_store_update_from_group (config, self);
	result = TRUE;
	(config == NULL) ? NULL : (config = (g_object_unref (config), NULL));
	return result;
}


/**
         * @returns: Object path to the ChannelList service for this device
         */
static char* dvb_device_group_real_GetChannelList (DVBIDBusDeviceGroup* base) {
	DVBDeviceGroup * self;
	char* result;
	char* _tmp0_;
	char* _tmp1_;
	self = (DVBDeviceGroup*) base;
	_tmp0_ = NULL;
	_tmp1_ = NULL;
	result = (_tmp1_ = g_strdup (_tmp0_ = g_strdup_printf (DVB_CONSTANTS_DBUS_CHANNEL_LIST_PATH, self->priv->_Id)), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
	return result;
}


gboolean dvb_device_group_register_channel_list (DVBDeviceGroup* self) {
	gboolean result;
	DVBChannelList* _tmp0_;
	DVBChannelList* channels;
	DBusGConnection* conn;
	char* path;
	g_return_val_if_fail (self != NULL, FALSE);
	g_debug ("DeviceGroup.vala:322: Creating new ChannelList D-Bus service for group %u", self->priv->_Id);
	_tmp0_ = NULL;
	channels = (_tmp0_ = dvb_device_group_get_Channels (self), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	conn = dvb_utils_get_dbus_connection ();
	if (conn == NULL) {
		result = FALSE;
		(channels == NULL) ? NULL : (channels = (g_object_unref (channels), NULL));
		(conn == NULL) ? NULL : (conn = (dbus_g_connection_unref (conn), NULL));
		return result;
	}
	path = g_strdup_printf (DVB_CONSTANTS_DBUS_CHANNEL_LIST_PATH, self->priv->_Id);
	_vala_dbus_register_object (dbus_g_connection_get_connection (conn), path, (GObject*) channels);
	result = TRUE;
	(channels == NULL) ? NULL : (channels = (g_object_unref (channels), NULL));
	(conn == NULL) ? NULL : (conn = (dbus_g_connection_unref (conn), NULL));
	path = (g_free (path), NULL);
	return result;
}


/**
         * @returns: List of paths to the devices that are part of
         * the group (e.g. /dev/dvb/adapter0/frontend0)
         */
static char** dvb_device_group_real_GetMembers (DVBIDBusDeviceGroup* base, int* result_length1) {
	DVBDeviceGroup * self;
	char** result;
	char** _tmp1_;
	gint groupdevs_size;
	gint groupdevs_length1;
	gint _tmp0_;
	char** groupdevs;
	gint i;
	char** _tmp3_;
	self = (DVBDeviceGroup*) base;
	_tmp1_ = NULL;
	groupdevs = (_tmp1_ = g_new0 (char*, (_tmp0_ = dvb_device_group_get_size (self)) + 1), groupdevs_length1 = _tmp0_, groupdevs_size = groupdevs_length1, _tmp1_);
	i = 0;
	g_static_rec_mutex_lock (&self->priv->__lock_devices);
	{
		{
			GeeIterator* _dev_it;
			_dev_it = gee_iterable_iterator ((GeeIterable*) self->priv->devices);
			while (TRUE) {
				DVBDevice* dev;
				char* _tmp2_;
				if (!gee_iterator_next (_dev_it)) {
					break;
				}
				dev = (DVBDevice*) gee_iterator_get (_dev_it);
				_tmp2_ = NULL;
				groupdevs[i] = (_tmp2_ = g_strdup_printf (DVB_CONSTANTS_DVB_DEVICE_PATH, dvb_device_get_Adapter (dev), dvb_device_get_Frontend (dev)), groupdevs[i] = (g_free (groupdevs[i]), NULL), _tmp2_);
				i++;
				(dev == NULL) ? NULL : (dev = (g_object_unref (dev), NULL));
			}
			(_dev_it == NULL) ? NULL : (_dev_it = (g_object_unref (_dev_it), NULL));
		}
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_devices);
	_tmp3_ = NULL;
	result = (_tmp3_ = groupdevs, *result_length1 = groupdevs_length1, _tmp3_);
	return result;
	groupdevs = (_vala_array_free (groupdevs, groupdevs_length1, (GDestroyNotify) g_free), NULL);
}


/**
         * @channel_sid: ID of the channel
         * @returns: Object path to Schedule service
         */
static char* dvb_device_group_real_GetSchedule (DVBIDBusDeviceGroup* base, guint channel_sid) {
	DVBDeviceGroup * self;
	char* result;
	self = (DVBDeviceGroup*) base;
	if (dvb_channel_list_contains (dvb_device_group_get_Channels (self), channel_sid)) {
		char* path;
		path = g_strdup_printf (DVB_CONSTANTS_DBUS_SCHEDULE_PATH, self->priv->_Id, channel_sid);
		if (!gee_collection_contains ((GeeCollection*) self->priv->schedules, path)) {
			DBusGConnection* conn;
			DVBSchedule* _tmp2_;
			DVBChannel* _tmp1_;
			DVBSchedule* _tmp3_;
			DVBSchedule* schedule;
			conn = dvb_utils_get_dbus_connection ();
			if (conn == NULL) {
				char* _tmp0_;
				_tmp0_ = NULL;
				_tmp0_ = g_strdup ("");
				_tmp0_ = (g_free (_tmp0_), NULL);
			}
			_tmp2_ = NULL;
			_tmp1_ = NULL;
			_tmp3_ = NULL;
			schedule = (_tmp3_ = (_tmp2_ = dvb_channel_get_Schedule (_tmp1_ = dvb_channel_list_get_channel (dvb_device_group_get_Channels (self), channel_sid)), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL)), _tmp3_);
			_vala_dbus_register_object (dbus_g_connection_get_connection (conn), path, (GObject*) schedule);
			gee_collection_add ((GeeCollection*) self->priv->schedules, path);
			(conn == NULL) ? NULL : (conn = (dbus_g_connection_unref (conn), NULL));
			(schedule == NULL) ? NULL : (schedule = (g_object_unref (schedule), NULL));
		}
		result = g_strdup (path);
		path = (g_free (path), NULL);
		return result;
	}
	result = g_strdup ("");
	return result;
}


/**
         * @returns: Location of the recordings directory
         */
static char* dvb_device_group_real_GetRecordingsDirectory (DVBIDBusDeviceGroup* base) {
	DVBDeviceGroup * self;
	char* result;
	self = (DVBDeviceGroup*) base;
	result = g_file_get_path (dvb_device_group_get_RecordingsDirectory (self));
	return result;
}


/**
         * @location: Location of the recordings directory
         * @returns: TRUE on success
         */
static gboolean dvb_device_group_real_SetRecordingsDirectory (DVBIDBusDeviceGroup* base, const char* location) {
	DVBDeviceGroup * self;
	gboolean result;
	GFile* _tmp0_;
	DVBConfigStore* _tmp1_;
	DVBConfigStore* config;
	self = (DVBDeviceGroup*) base;
	g_return_val_if_fail (location != NULL, FALSE);
	_tmp0_ = NULL;
	dvb_device_group_set_RecordingsDirectory (self, _tmp0_ = g_file_new_for_path (location));
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
	_tmp1_ = NULL;
	config = (_tmp1_ = dvb_factory_get_config_store (), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_));
	dvb_config_store_update_from_group (config, self);
	result = TRUE;
	(config == NULL) ? NULL : (config = (g_object_unref (config), NULL));
	return result;
}


static GType dvb_device_group_real_get_element_type (GeeIterable* base) {
	DVBDeviceGroup * self;
	GType result;
	self = (DVBDeviceGroup*) base;
	result = DVB_TYPE_DEVICE;
	return result;
}


static GeeIterator* dvb_device_group_real_iterator (GeeIterable* base) {
	DVBDeviceGroup * self;
	GeeIterator* result;
	self = (DVBDeviceGroup*) base;
	result = gee_iterable_iterator ((GeeIterable*) self->priv->devices);
	return result;
}


/**
         * Set RecordingsDirectory property of all
         * devices to the values of the reference device
         */
static void dvb_device_group_update_all_devices (DVBDeviceGroup* self) {
	g_return_if_fail (self != NULL);
	g_static_rec_mutex_lock (&self->priv->__lock_devices);
	{
		{
			GeeIterator* _device_it;
			_device_it = gee_iterable_iterator ((GeeIterable*) self->priv->devices);
			while (TRUE) {
				DVBDevice* device;
				if (!gee_iterator_next (_device_it)) {
					break;
				}
				device = (DVBDevice*) gee_iterator_get (_device_it);
				if (device != self->priv->_reference_device) {
					dvb_device_set_RecordingsDirectory (device, dvb_device_get_RecordingsDirectory (self->priv->_reference_device));
				}
				(device == NULL) ? NULL : (device = (g_object_unref (device), NULL));
			}
			(_device_it == NULL) ? NULL : (_device_it = (g_object_unref (_device_it), NULL));
		}
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_devices);
}


gint dvb_device_group_get_size (DVBDeviceGroup* self) {
	gint result;
	g_return_val_if_fail (self != NULL, 0);
	result = gee_collection_get_size ((GeeCollection*) self->priv->devices);
	return result;
}


guint dvb_device_group_get_Id (DVBDeviceGroup* self) {
	guint result;
	g_return_val_if_fail (self != NULL, 0U);
	result = self->priv->_Id;
	return result;
}


static void dvb_device_group_set_Id (DVBDeviceGroup* self, guint value) {
	g_return_if_fail (self != NULL);
	self->priv->_Id = value;
	g_object_notify ((GObject *) self, "Id");
}


DVBChannelList* dvb_device_group_get_Channels (DVBDeviceGroup* self) {
	DVBChannelList* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = dvb_device_get_Channels (self->priv->_reference_device);
	return result;
}


GFile* dvb_device_group_get_RecordingsDirectory (DVBDeviceGroup* self) {
	GFile* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = dvb_device_get_RecordingsDirectory (self->priv->_reference_device);
	return result;
}


void dvb_device_group_set_RecordingsDirectory (DVBDeviceGroup* self, GFile* value) {
	g_return_if_fail (self != NULL);
	dvb_device_set_RecordingsDirectory (self->priv->_reference_device, value);
	dvb_device_group_update_all_devices (self);
	g_object_notify ((GObject *) self, "RecordingsDirectory");
}


DVBAdapterType dvb_device_group_get_Type (DVBDeviceGroup* self) {
	DVBAdapterType result;
	g_return_val_if_fail (self != NULL, 0);
	result = dvb_device_get_Type (self->priv->_reference_device);
	return result;
}


DVBRecorder* dvb_device_group_get_recorder (DVBDeviceGroup* self) {
	DVBRecorder* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_recorder;
	return result;
}


DVBEPGScanner* dvb_device_group_get_epgscanner (DVBDeviceGroup* self) {
	DVBEPGScanner* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_epgscanner;
	return result;
}


DVBChannelFactory* dvb_device_group_get_channel_factory (DVBDeviceGroup* self) {
	DVBChannelFactory* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_channelfactory;
	return result;
}


const char* dvb_device_group_get_Name (DVBDeviceGroup* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_Name;
	return result;
}


void dvb_device_group_set_Name (DVBDeviceGroup* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->_Name = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_Name = (g_free (self->priv->_Name), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "Name");
}


DVBDevice* dvb_device_group_get_reference_device (DVBDeviceGroup* self) {
	DVBDevice* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_reference_device;
	return result;
}


static void dvb_device_group_set_reference_device (DVBDeviceGroup* self, DVBDevice* value) {
	DVBDevice* _tmp1_;
	DVBDevice* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->_reference_device = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_reference_device == NULL) ? NULL : (self->priv->_reference_device = (g_object_unref (self->priv->_reference_device), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "reference-device");
}


static guint _dvb_device_hash_ghash_func (void* key) {
	return dvb_device_hash (key);
}


static gboolean _dvb_device_equal_gequal_func (void* a, void* b) {
	return dvb_device_equal (a, b);
}


static GObject * dvb_device_group_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	DVBDeviceGroupClass * klass;
	GObjectClass * parent_class;
	DVBDeviceGroup * self;
	klass = DVB_DEVICE_GROUP_CLASS (g_type_class_peek (DVB_TYPE_DEVICE_GROUP));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = DVB_DEVICE_GROUP (obj);
	{
		GeeSet* _tmp0_;
		GeeHashSet* _tmp1_;
		DVBChannelFactory* _tmp2_;
		DVBRecorder* _tmp3_;
		_tmp0_ = NULL;
		self->priv->devices = (_tmp0_ = (GeeSet*) gee_hash_set_new (DVB_TYPE_DEVICE, (GBoxedCopyFunc) g_object_ref, g_object_unref, _dvb_device_hash_ghash_func, _dvb_device_equal_gequal_func), (self->priv->devices == NULL) ? NULL : (self->priv->devices = (g_object_unref (self->priv->devices), NULL)), _tmp0_);
		_tmp1_ = NULL;
		self->priv->schedules = (_tmp1_ = gee_hash_set_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_str_hash, g_str_equal), (self->priv->schedules == NULL) ? NULL : (self->priv->schedules = (g_object_unref (self->priv->schedules), NULL)), _tmp1_);
		gee_collection_add ((GeeCollection*) self->priv->devices, self->priv->_reference_device);
		_tmp2_ = NULL;
		self->priv->_channelfactory = (_tmp2_ = dvb_channel_factory_new (self), (self->priv->_channelfactory == NULL) ? NULL : (self->priv->_channelfactory = (g_object_unref (self->priv->_channelfactory), NULL)), _tmp2_);
		_tmp3_ = NULL;
		self->priv->_recorder = (_tmp3_ = dvb_recorder_new (self), (self->priv->_recorder == NULL) ? NULL : (self->priv->_recorder = (g_object_unref (self->priv->_recorder), NULL)), _tmp3_);
	}
	return obj;
}


void _dvb_device_group_dbus_unregister (DBusConnection* connection, void* user_data) {
}


static DBusMessage* _dbus_dvb_device_group_introspect (DVBDeviceGroup* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessage* reply;
	DBusMessageIter iter;
	GString* xml_data;
	char** children;
	int i;
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	xml_data = g_string_new ("<!DOCTYPE node PUBLIC \"-//freedesktop//DTD D-BUS Object Introspection 1.0//EN\" \"http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd\">\n");
	g_string_append (xml_data, "<node>\n<interface name=\"org.freedesktop.DBus.Introspectable\">\n  <method name=\"Introspect\">\n    <arg name=\"data\" direction=\"out\" type=\"s\"/>\n  </method>\n</interface>\n<interface name=\"org.freedesktop.DBus.Properties\">\n  <method name=\"Get\">\n    <arg name=\"interface\" direction=\"in\" type=\"s\"/>\n    <arg name=\"propname\" direction=\"in\" type=\"s\"/>\n    <arg name=\"value\" direction=\"out\" type=\"v\"/>\n  </method>\n  <method name=\"Set\">\n    <arg name=\"interface\" direction=\"in\" type=\"s\"/>\n    <arg name=\"propname\" direction=\"in\" type=\"s\"/>\n    <arg name=\"value\" direction=\"in\" type=\"v\"/>\n  </method>\n  <method name=\"GetAll\">\n    <arg name=\"interface\" direction=\"in\" type=\"s\"/>\n    <arg name=\"props\" direction=\"out\" type=\"a{sv}\"/>\n  </method>\n</interface>\n<interface name=\"org.gnome.DVB.DeviceGroup\">\n  <method name=\"GetType\">\n    <arg name=\"result\" type=\"s\" direction=\"out\"/>\n  </method>\n  <method name=\"GetRecorder\">\n    <arg name=\"result\" type=\"o\" direction=\"out\"/>\n  </method>\n  <method name=\"AddDevice\">\n    <arg name=\"adapter\" type=\"u\" direction=\"in\"/>\n    <arg name=\"frontend\" type=\"u\" direction=\"in\"/>\n    <arg name=\"result\" type=\"b\" direction=\"out\"/>\n  </method>\n  <method name=\"RemoveDevice\">\n    <arg name=\"adapter\" type=\"u\" direction=\"in\"/>\n    <arg name=\"frontend\" type=\"u\" direction=\"in\"/>\n    <arg name=\"result\" type=\"b\" direction=\"out\"/>\n  </method>\n  <method name=\"GetChannelList\">\n    <arg name=\"result\" type=\"o\" direction=\"out\"/>\n  </method>\n  <method name=\"GetName\">\n    <arg name=\"result\" type=\"s\" direction=\"out\"/>\n  </method>\n  <method name=\"SetName\">\n    <arg name=\"name\" type=\"s\" direction=\"in\"/>\n    <arg name=\"result\" type=\"b\" direction=\"out\"/>\n  </method>\n  <method name=\"GetMembers\">\n    <arg name=\"result\" type=\"as\" direction=\"out\"/>\n  </method>\n  <method name=\"GetSchedule\">\n    <arg name=\"channel_sid\" type=\"u\" direction=\"in\"/>\n    <arg name=\"result\" type=\"o\" direction=\"out\"/>\n  </method>\n  <method name=\"GetRecordingsDirectory\">\n    <arg name=\"result\" type=\"s\" direction=\"out\"/>\n  </method>\n  <method name=\"SetRecordingsDirectory\">\n    <arg name=\"location\" type=\"s\" direction=\"in\"/>\n    <arg name=\"result\" type=\"b\" direction=\"out\"/>\n  </method>\n  <signal name=\"DeviceAdded\">\n    <arg name=\"adapter\" type=\"u\"/>\n    <arg name=\"frontend\" type=\"u\"/>\n  </signal>\n  <signal name=\"DeviceRemoved\">\n    <arg name=\"adapter\" type=\"u\"/>\n    <arg name=\"frontend\" type=\"u\"/>\n  </signal>\n</interface>\n");
	dbus_connection_list_registered (connection, g_object_get_data ((GObject *) self, "dbus_object_path"), &children);
	for (i = 0; children[i]; i++) {
		g_string_append_printf (xml_data, "<node name=\"%s\"/>\n", children[i]);
	}
	dbus_free_string_array (children);
	g_string_append (xml_data, "</node>\n");
	dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &xml_data->str);
	g_string_free (xml_data, TRUE);
	return reply;
}


DBusHandlerResult dvb_device_group_dbus_message (DBusConnection* connection, DBusMessage* message, void* object) {
	DBusMessage* reply;
	reply = NULL;
	if (dbus_message_is_method_call (message, "org.freedesktop.DBus.Introspectable", "Introspect")) {
		reply = _dbus_dvb_device_group_introspect (object, connection, message);
	}
	if (reply) {
		dbus_connection_send (connection, reply, NULL);
		dbus_message_unref (reply);
		return DBUS_HANDLER_RESULT_HANDLED;
	} else if (dvb_id_bus_device_group_dbus_message (connection, message, object) == DBUS_HANDLER_RESULT_HANDLED) {
		return DBUS_HANDLER_RESULT_HANDLED;
	} else {
		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
	}
}


void dvb_device_group_dbus_register_object (DBusConnection* connection, const char* path, void* object) {
	if (!g_object_get_data (object, "dbus_object_path")) {
		g_object_set_data (object, "dbus_object_path", g_strdup (path));
		dbus_connection_register_object_path (connection, path, &_dvb_device_group_dbus_path_vtable, object);
		g_object_weak_ref (object, _vala_dbus_unregister_object, connection);
	}
	dvb_id_bus_device_group_dbus_register_object (connection, path, object);
}


static void dvb_device_group_class_init (DVBDeviceGroupClass * klass) {
	dvb_device_group_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (DVBDeviceGroupPrivate));
	G_OBJECT_CLASS (klass)->get_property = dvb_device_group_get_property;
	G_OBJECT_CLASS (klass)->set_property = dvb_device_group_set_property;
	G_OBJECT_CLASS (klass)->constructor = dvb_device_group_constructor;
	G_OBJECT_CLASS (klass)->finalize = dvb_device_group_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_DEVICE_GROUP_SIZE, g_param_spec_int ("size", "size", "size", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_DEVICE_GROUP_ID, g_param_spec_uint ("Id", "Id", "Id", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_DEVICE_GROUP_CHANNELS, g_param_spec_object ("Channels", "Channels", "Channels", DVB_TYPE_CHANNEL_LIST, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_DEVICE_GROUP_RECORDINGS_DIRECTORY, g_param_spec_object ("RecordingsDirectory", "RecordingsDirectory", "RecordingsDirectory", G_TYPE_FILE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_DEVICE_GROUP_TYPE, g_param_spec_enum ("Type", "Type", "Type", DVB_TYPE_ADAPTER_TYPE, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_DEVICE_GROUP_RECORDER, g_param_spec_object ("recorder", "recorder", "recorder", DVB_TYPE_RECORDER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_DEVICE_GROUP_EPGSCANNER, g_param_spec_object ("epgscanner", "epgscanner", "epgscanner", DVB_TYPE_EPG_SCANNER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_DEVICE_GROUP_CHANNEL_FACTORY, g_param_spec_object ("channel-factory", "channel-factory", "channel-factory", DVB_TYPE_CHANNEL_FACTORY, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_DEVICE_GROUP_NAME, g_param_spec_string ("Name", "Name", "Name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_DEVICE_GROUP_REFERENCE_DEVICE, g_param_spec_object ("reference-device", "reference-device", "reference-device", DVB_TYPE_DEVICE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_type_set_qdata (DVB_TYPE_DEVICE_GROUP, g_quark_from_static_string ("DBusObjectVTable"), (void*) (&_dvb_device_group_dbus_vtable));
}


static void dvb_device_group_dvb_id_bus_device_group_interface_init (DVBIDBusDeviceGroupIface * iface) {
	dvb_device_group_dvb_id_bus_device_group_parent_iface = g_type_interface_peek_parent (iface);
	iface->GetType = dvb_device_group_real_GetType;
	iface->AddDevice = dvb_device_group_real_AddDevice;
	iface->GetRecorder = dvb_device_group_real_GetRecorder;
	iface->RemoveDevice = dvb_device_group_real_RemoveDevice;
	iface->GetName = dvb_device_group_real_GetName;
	iface->SetName = dvb_device_group_real_SetName;
	iface->GetChannelList = dvb_device_group_real_GetChannelList;
	iface->GetMembers = dvb_device_group_real_GetMembers;
	iface->GetSchedule = dvb_device_group_real_GetSchedule;
	iface->GetRecordingsDirectory = dvb_device_group_real_GetRecordingsDirectory;
	iface->SetRecordingsDirectory = dvb_device_group_real_SetRecordingsDirectory;
}


static void dvb_device_group_gee_iterable_interface_init (GeeIterableIface * iface) {
	dvb_device_group_gee_iterable_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_element_type = dvb_device_group_real_get_element_type;
	iface->iterator = dvb_device_group_real_iterator;
}


static void dvb_device_group_instance_init (DVBDeviceGroup * self) {
	self->priv = DVB_DEVICE_GROUP_GET_PRIVATE (self);
	g_static_rec_mutex_init (&self->priv->__lock_devices);
}


static void dvb_device_group_finalize (GObject* obj) {
	DVBDeviceGroup * self;
	self = DVB_DEVICE_GROUP (obj);
	self->priv->_Name = (g_free (self->priv->_Name), NULL);
	(self->priv->_reference_device == NULL) ? NULL : (self->priv->_reference_device = (g_object_unref (self->priv->_reference_device), NULL));
	g_static_rec_mutex_free (&self->priv->__lock_devices);
	(self->priv->devices == NULL) ? NULL : (self->priv->devices = (g_object_unref (self->priv->devices), NULL));
	(self->priv->_recorder == NULL) ? NULL : (self->priv->_recorder = (g_object_unref (self->priv->_recorder), NULL));
	(self->priv->_epgscanner == NULL) ? NULL : (self->priv->_epgscanner = (g_object_unref (self->priv->_epgscanner), NULL));
	(self->priv->_channelfactory == NULL) ? NULL : (self->priv->_channelfactory = (g_object_unref (self->priv->_channelfactory), NULL));
	(self->priv->schedules == NULL) ? NULL : (self->priv->schedules = (g_object_unref (self->priv->schedules), NULL));
	G_OBJECT_CLASS (dvb_device_group_parent_class)->finalize (obj);
}


GType dvb_device_group_get_type (void) {
	static GType dvb_device_group_type_id = 0;
	if (dvb_device_group_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (DVBDeviceGroupClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) dvb_device_group_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DVBDeviceGroup), 0, (GInstanceInitFunc) dvb_device_group_instance_init, NULL };
		static const GInterfaceInfo dvb_id_bus_device_group_info = { (GInterfaceInitFunc) dvb_device_group_dvb_id_bus_device_group_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo gee_iterable_info = { (GInterfaceInitFunc) dvb_device_group_gee_iterable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		dvb_device_group_type_id = g_type_register_static (G_TYPE_OBJECT, "DVBDeviceGroup", &g_define_type_info, 0);
		g_type_add_interface_static (dvb_device_group_type_id, DVB_TYPE_ID_BUS_DEVICE_GROUP, &dvb_id_bus_device_group_info);
		g_type_add_interface_static (dvb_device_group_type_id, GEE_TYPE_ITERABLE, &gee_iterable_info);
	}
	return dvb_device_group_type_id;
}


static void dvb_device_group_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	DVBDeviceGroup * self;
	gpointer boxed;
	self = DVB_DEVICE_GROUP (object);
	switch (property_id) {
		case DVB_DEVICE_GROUP_SIZE:
		g_value_set_int (value, dvb_device_group_get_size (self));
		break;
		case DVB_DEVICE_GROUP_ID:
		g_value_set_uint (value, dvb_device_group_get_Id (self));
		break;
		case DVB_DEVICE_GROUP_CHANNELS:
		g_value_set_object (value, dvb_device_group_get_Channels (self));
		break;
		case DVB_DEVICE_GROUP_RECORDINGS_DIRECTORY:
		g_value_set_object (value, dvb_device_group_get_RecordingsDirectory (self));
		break;
		case DVB_DEVICE_GROUP_TYPE:
		g_value_set_enum (value, dvb_device_group_get_Type (self));
		break;
		case DVB_DEVICE_GROUP_RECORDER:
		g_value_set_object (value, dvb_device_group_get_recorder (self));
		break;
		case DVB_DEVICE_GROUP_EPGSCANNER:
		g_value_set_object (value, dvb_device_group_get_epgscanner (self));
		break;
		case DVB_DEVICE_GROUP_CHANNEL_FACTORY:
		g_value_set_object (value, dvb_device_group_get_channel_factory (self));
		break;
		case DVB_DEVICE_GROUP_NAME:
		g_value_set_string (value, dvb_device_group_get_Name (self));
		break;
		case DVB_DEVICE_GROUP_REFERENCE_DEVICE:
		g_value_set_object (value, dvb_device_group_get_reference_device (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void dvb_device_group_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	DVBDeviceGroup * self;
	self = DVB_DEVICE_GROUP (object);
	switch (property_id) {
		case DVB_DEVICE_GROUP_ID:
		dvb_device_group_set_Id (self, g_value_get_uint (value));
		break;
		case DVB_DEVICE_GROUP_RECORDINGS_DIRECTORY:
		dvb_device_group_set_RecordingsDirectory (self, g_value_get_object (value));
		break;
		case DVB_DEVICE_GROUP_NAME:
		dvb_device_group_set_Name (self, g_value_get_string (value));
		break;
		case DVB_DEVICE_GROUP_REFERENCE_DEVICE:
		dvb_device_group_set_reference_device (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static void _vala_dbus_register_object (DBusConnection* connection, const char* path, void* object) {
	const _DBusObjectVTable * vtable;
	vtable = g_type_get_qdata (G_TYPE_FROM_INSTANCE (object), g_quark_from_static_string ("DBusObjectVTable"));
	if (vtable) {
		vtable->register_object (connection, path, object);
	} else {
		g_warning ("Object does not implement any D-Bus interface");
	}
}


static void _vala_dbus_unregister_object (gpointer connection, GObject* object) {
	char* path;
	path = g_object_steal_data ((GObject*) object, "dbus_object_path");
	dbus_connection_unregister_object_path (connection, path);
	g_free (path);
}




